@@ -71,6 +71,9 @@ def prepare_date(data, schema):
     """Converts datetime.date to int timestamp"""
     if isinstance(data, datetime.date):
         return data.toordinal() - DAYS_SHIFT
+    elif isinstance(data, str):
+        days = datetime.datetime.strptime(data, "%Y-%m-%d").toordinal()
+        return days - DAYS_SHIFT
     else:
         return data
 
@@ -324,7 +327,11 @@ def write_union(fo, datum, schema):
         for index, candidate in enumerate(schema):
             if validate(datum, candidate, raise_errors=False):
                 if extract_record_type(candidate) == 'record':
-                    fields = len(candidate['fields'])
+                    candidate_fields = set(
+                        f["name"] for f in candidate["fields"]
+                    )
+                    datum_fields = set(datum)
+                    fields = len(candidate_fields.intersection(datum_fields))
                     if fields > most_fields:
                         best_match_index = index
                         most_fields = fields
@@ -501,6 +508,14 @@ class Writer(object):
 
             self.metadata = metadata or {}
             self.metadata['avro.codec'] = codec
+
+            if isinstance(schema, dict):
+                schema = {
+                    key: value
+                    for key, value in iteritems(schema)
+                    if key != "__fastavro_parsed"
+                }
+
             self.metadata['avro.schema'] = json.dumps(schema)
 
             try:
@@ -526,6 +541,14 @@ class Writer(object):
         if self.io.tell() >= self.sync_interval:
             self.dump()
 
+    def write_block(self, block):
+        # Clear existing block if there are any records pending
+        if self.io.tell() or self.block_count > 0:
+            self.dump()
+        write_long(self.fo, block.num_records)
+        self.block_writer(self.fo, block.bytes_.getvalue())
+        self.fo.write(self.sync_marker)
+
     def flush(self):
         if self.io.tell() or self.block_count > 0:
             self.dump()
